'use strict';
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchMetaData = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const cheerio = __importStar(require("cheerio"));
const util_1 = require("./util");
/**
 *  Fetches meta data of a given website url
 * @param url | the website url to fetch the metadata from
 */
exports.fetchMetaData = async (url, _options) => {
    var _a, _b;
    try {
        const urlString = url.trim();
        // Options validation
        if (!_options || typeof _options !== 'object') {
            _options = {};
        }
        const options = {
            userAgent: (_a = _options.userAgent) !== null && _a !== void 0 ? _a : 'meta-fetch',
            fromEmail: (_b = _options.userAgent) !== null && _b !== void 0 ? _b : 'metafetch@email.com'
        };
        const requestOptions = {
            method: 'GET',
            headers: {
                'User-Agent': options.userAgent,
                'From': options.fromEmail
            }
        };
        const response = await node_fetch_1.default(urlString, requestOptions);
        const content = await response.text();
        // Load html to cheerio
        const $ = cheerio.load(content);
        const head = $('head');
        // Basic site meta-data
        const basicMeta = () => {
            const website = response.url;
            const title = head.find('title').text();
            const desc = head.find('meta[name=description]').attr('content');
            return {
                website,
                title,
                description: desc
            };
        };
        // Open graph basic
        const fetchMeta = () => {
            const openGraphsArray = head.find('meta[property]');
            const openGraphs = {};
            openGraphsArray.each((_, element) => {
                const property = $(element)
                    .attr('property');
                const content = $(element).attr('content');
                if (!property.includes('twitter')) {
                    openGraphs[property] = content;
                }
            });
            return openGraphs;
        };
        // Open graph social
        const fetchMetaSocial = () => {
            const openGraphsArray = head.find('meta[name]');
            const socials = {};
            openGraphsArray.each((_, element) => {
                const property = $(element).attr('name');
                const content = $(element).attr('content');
                if (property.includes('twitter')) {
                    socials[property] = content;
                }
            });
            return socials;
        };
        // Favicons
        const fetchFavicons = () => {
            const faviconArray = head.find('link[rel]');
            const favicons = [];
            faviconArray.each((_, element) => {
                const href = $(element).attr('href');
                if (href.includes('shortcut icon') || href.includes('icon') || href.includes('apple-touch-startup-image') || href.includes('apple-touch-icon')) {
                    const validUri = util_1.createValidUri(response.url, href);
                    favicons.push(validUri);
                }
            });
            return favicons;
        };
        // Meta-data
        const basicMetaData = basicMeta();
        const openGraphs = fetchMeta();
        const openGraph_social = fetchMetaSocial();
        const favicons = fetchFavicons();
        const metaData = {
            basic_metadata: basicMetaData,
            opengraph: openGraphs,
            opengraph_social: openGraph_social,
            favicons
        };
        return metaData;
    }
    catch (error) {
        console.error(error);
    }
};
